function [K,Kinit,Kff,nmeas,nu,P,Pt,Pa,Pbt,outputs_a] = ...
            gen_interconnect(claw_syn,t_indi,t_actfb_mdl,P,Pa,H_fb, ...
                              H_aa,H_syn,H_alloc,H_act,inputs,outputs_a)

%   GEN_INTERCONNECT Generate IDI generalized control interconnection structure
%       Update control interconnection P for a (I)DI control law K of given
%       type (inversion strategy) and synchronization strategy (sync/async, 
%       actuator feedback mode). Use is made of the following (standard)
%       generalized interconnection structure:
%       
%                  +-----[  Δ  ]<---+  
%                  |                |
%                  |     +-----+    |
%                  O---->|     |----+
%            w --------->|  P  |--------> z
%                  O---->|     |----+
%                  |     +-----+    |
%                  |                |
%                  +-----[  K  ]<---+
%
%       Author: T.S.C. Pollack
%       Copyright (2024) T.S.C. Pollack

    % Set I/O names
    H_fb.InputName         = 'qserr';
    H_fb.OutputName        = 'nu';
    H_aa.InputName         = {'qdots','alphas','qs','ufb0'};
    H_aa.OutputName        = 'qdote';
    H_syn.InputName        = 'ufb0';
    H_syn.OutputName       = 'desyn';
    H_alloc.InputName      = 'dqdot';
    H_alloc.OutputName     = 'dde';
    
    K_inputs  = {'yqref','yqdotref','alphas','qs','qdots','ufb0'};
    K_outputs = {'u'};

    % Add summation blocks
    sum_qserr = sumblk('qserr = yqref - qs');
    sum_dqdot = sumblk('dqdot = yqdotref + nu - qdote');
    sum_u     = sumblk('u = dde + desyn');

    % Set CLAW I/O dimensions
    nmeas = 6; nu = 1;

    % Connect
    K = minreal(connect(H_fb, H_aa, H_syn, H_alloc, sum_qserr, ...
                        sum_dqdot, sum_u, K_inputs, K_outputs));  

    % Save controller for later reference
    Kinit = K; 
    
    % Initialize parameters
    Kff = tf(0);
    Pt  = P;
    Pbt = P;

    %% 4) Inversion filter synthesis: asynchronized
    if claw_syn == 4

        inputs_t = inputs;

        H_fb.InputName         = 'qserr';
        H_fb.OutputName        = 'nu';
        H_alloc.InputName      = 'uff_ba';
        H_alloc.OutputName     = 'uff';

        sum_qserr = sumblk('qserr = yqref - qs');
        sum_uff   = sumblk('uff_ba = yqdotref + nu');

        Kff_inputs  = {'yqref','yqdotref','qs'};
        Kff_outputs = {'uff'};

        % Connect
        Kff = minreal(connect(H_fb, H_alloc, sum_qserr, sum_uff, Kff_inputs, Kff_outputs));

        sum_u = sumblk('u = uff + ufb');
        inputs = {'udel','w','ufb'};  

        if strcmp(t_indi,'sb')

            outputs   = {'ydel','yqte','ynzte','yqdotte','qdots','ufb0'};                  
            outputs_a = {'ydel','yqte','ynzte','yqdotte','alpha','q','nz','qdot', ...
                            'qdots','ufb0'};
            outputs_b = outputs;

            nmeas = 2; nu = 1;

        elseif strcmp(t_indi,'mb')

            outputs   = {'ydel','yqte','ynzte','yqdotte','qdote','ufb0'};                  
            outputs_a = {'ydel','yqte','ynzte','yqdotte','alpha','q','nz','qdot', ...
                            'qdote','ufb0'};
            outputs_b = {'ydel','yqte','ynzte','yqdotte','alphas','qs','ufb0','ufb0'};    

            nmeas = 2; nu = 1;

        elseif strcmp(t_indi,'hb')

            outputs    = {'ydel','yqte','ynzte','yqdotte','qdote','qdots','ufb0'};                  
            outputs_a  = {'ydel','yqte','ynzte','yqdotte','alpha','q','nz','qdot', ...
                            'qdote','qdots','ufb0'};               
            outputs_b  = {'ydel','yqte','ynzte','yqdotte','alphas','qs','ufb0', ...
                            'qdots','ufb0'};

            nmeas = 3; nu = 1;

        end

        Pt  = connect(P,H_aa,inputs_t,[Kff_inputs,outputs(5:end)]);
        Pbt = connect(P,H_aa,inputs_t,[Kff_inputs,outputs_b(5:end)]);

        P  = connect(P,Kff,H_aa,sum_u,inputs,outputs);
        Pa = connect(Pa,Kff,H_aa,sum_u,inputs,outputs_a);            
    
    %% 6) Inversion filter synthesis: synchronized (filtering equivalence) 
    elseif claw_syn == 6
        
        inputs_t = inputs;

        H_fb.InputName         = 'qserr';
        H_fb.OutputName        = 'nu';
        H_alloc.InputName      = 'uff_ba';
        H_alloc.OutputName     = 'uff';

        sum_qserr = sumblk('qserr = yqref - qs');
        sum_uff   = sumblk('uff_ba = yqdotref + nu');

        Kff_inputs  = {'yqref','yqdotref','qs'};
        Kff_outputs = {'uff'};

        % Connect
        Kff = minreal(connect(H_fb, H_alloc, sum_qserr, sum_uff, Kff_inputs, Kff_outputs));

        sum_u = sumblk('u = uff + ufb');
        inputs = {'udel','w','ufb'};  

        if strcmp(t_indi,'sb')
            
            H_alloc.InputName      = 'qdots';
            H_alloc.OutputName     = 'qdots_hat';
            
            if t_actfb_mdl
                H_act_obm = H_act;
            else
                H_act_obm = tf(1);
            end
            
            H_act_obm.InputName  = 'ufb0';
            H_act_obm.OutputName = 'ufbmdl';    

            sum_inv_fb = sumblk('inv_fb = -qdots_hat + ufbmdl');            

            outputs   = {'ydel','yqte','ynzte','yqdotte','inv_fb'};                  
            outputs_a = {'ydel','yqte','ynzte','yqdotte','alpha','q','nz','qdot', ...
                            'inv_fb'};                
            outputs_b = {'ydel','yqte','ynzte','yqdotte','qdots','ufb0'};

            nmeas = 1; nu = 1;

        elseif strcmp(t_indi,'mb')

            fprintf('CLAW setting is not compatible with this synthesis mode');
            return;

        elseif strcmp(t_indi,'hb')

            fprintf('CLAW setting is not compatible with this synthesis mode');
            return;

        end

        Pt  = connect(P,H_aa,H_alloc,H_act_obm,sum_inv_fb,inputs_t,[Kff_inputs,outputs(5:end)]);
        Pbt = connect(P,H_aa,H_alloc,H_act_obm,sum_inv_fb,inputs_t,[Kff_inputs,outputs_b(5:end)]);

        P  = connect(P,Kff,H_aa,H_alloc,H_act_obm,sum_inv_fb,sum_u,inputs,outputs);
        Pa = connect(Pa,Kff,H_aa,H_alloc,H_act_obm,sum_inv_fb,sum_u,inputs,outputs_a);   
    end
    
end