function Lacl = act_connect(P,K,H_aa,dparam)
%   ACT_CONNECT Returns the broken-loop response with actuator
%       feedback connected for incremental control laws
%
%   Lacl = ACT_CONNECT(P,K,H_aa,dparam) takes the uncertain open-loop state 
%   space system P (see USS) and IDI-control law K in combination with
%   angular acceleration estimation scheme H_aa; dparam indicates whether
%   parametric uncertainty is present.
%
%       Author: T.S.C. Pollack
%       Copyright (2024) T.S.C. Pollack

    % Take nominal plant in case of parametric uncertainty
    if dparam
        P = P.NominalValue;
    end
    
    % Break down constructed signals
    qdote_bool = any(strcmp(K.Inputname,'qdote'));
    
    if qdote_bool
                
        % Expand qdot_e controller inputs
        H_aa = H_aa(2:4);
        
        H_aa.InputName         = {'alphas_e','qs_e','ufb0_e'};
        H_aa.OutputName        = 'qdote';
        
        qdote_idx = find(strcmp(K.Inputname,'qdote'));
        
        inputs = [K.InputName(1:qdote_idx-1);H_aa.InputName; ...
            K.InputName(qdote_idx+1:end)];
        
        K = connect(K,H_aa,inputs,K.OutputName);      
                
    end
    
    % Number of control law inputs
    nmeas = length(K);

    % Extract nominal performance block
    NP = size(P,1); Pr = P(NP-nmeas+1:NP,end);
    
    mask_idx = find(~contains(K.InputName,'ufb0'));
    
    % Mask signal loops that must remain broken
    K.Inputname(mask_idx) = {'x'};
    Pr.Outputname(mask_idx) = split(num2str(1:length(Pr(mask_idx,1))));
    
    % Rename associated qdot_e plant outputs
    if qdote_bool
        
        Pr.OutputName(qdote_idx+2) = {'ufb0_e'};
        
    end
    
    % Connect remaining loop
	Pr_acl = connect(Pr,K,K.OutputName,Pr.OutputName(mask_idx));
    
    % Return broken-loop transfer function
    Lacl = -K(mask_idx)*Pr_acl;
    
end

