%==========================================================================
% [plt_results.m] Perform mu-analysis and visualize results
%   Generates mu-diagrams and other plots of (synthesized) IDI-based 
%   control laws according to the following standard generalized system 
%   interconnection:
%       
%                  +-----[  Δ  ]<---+  
%                  |                |
%                  |     +-----+    |
%                  O---->|     |----+
%            w --------->|  P  |--------> z
%                  O---->|     |----+
%                  |     +-----+    |
%                  |                |
%                  +-----[  K  ]<---+
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

close all;

% Set frequency range
omega = omega_a;

% Set color list
clr_lst = [[0, 0.4470, 0.7410];[0.8500, 0.3250, 0.0980];[0.4660, 0.6740, 0.1880]; ...
    [0.4940, 0.1840, 0.5560];  [0.6350, 0.0780, 0.1840];[0.25, 0.25, 0.25]; ...
    [0.75, 0.75, 0];[0, 0.75, 0.75];[0.9290, 0.6940, 0.1250]];

% Get RP
M_rp = frd(lftdata(M),omega);
[mubnds_rp,~] = mussv(M_rp,[-1,0;-1,0;1,0;1,0;1,0;1,0;1,0;2,3],'a');

% --- (1) Plot RP mu-bounds --- 
FIG_muRP = figure(1); FIG_muRP.Position = [681 559 560 380];

semilogx(omega,mubnds_rp(:,1).ResponseData(:),'k-', ... 
    omega,mubnds_rp(:,2).ResponseData(:),'k--'); 

% Add legend
legend('$\bar{\mu}_{RP}$','$\underline{\mu}_{RP}$', ...
       'Location','NorthEast','Interpreter','latex');

grid on; hold on; xlim([omega(1),omega(end)]);

% Also plot full-order controller in case of post-synthesis order reduction
if fix_order >= 0 && strcmp(red_order,'postsyn')
    
    Mf = lft(P,Kfull); M_rpf = frd(lftdata(M),omega);
    [mubnds_rpf,~] = mussv(M_rpf,[-1,0;-1,0;1,0;1,0;1,0;1,0;1,0;2,3],'a');
    semilogx(omega,mubnds_rpf(:,1).ResponseData(:),'--', ...
        omega,mubnds_rpf(:,2).ResponseData(:),'--'); 
end

% Add axis labels
xlabel('Frequency (rad/s)','Interpreter','latex'); 
ylabel('$\mu$ (-)','Interpreter','latex'); 

% --- (2) M-analysis plot ---
FIG_muM = figure(2); FIG_muM.Position = [681 559 560 380];

% Mu for RP
semilogx(omega,mubnds_rp(:,1).ResponseData(:),'k-', ...
    omega,mubnds_rp(:,2).ResponseData(:),'k--'); 

grid on; hold on; xlim([omega(1),omega(end)]);

% Mu for RS
if dparam
    M_zv_g = frd(lftdata(M(1:length(Delta),1:length(Delta))),omega);
    [mubnds_rs,Info] = mussv(M_zv_g,[-1,0;-1,0;1,0;1,0;1,0;1,0;1,0],'a');
else
    M_zv_g = frd(M(1:length(Delta),1:length(Delta)),omega);
    [mubnds_rs,Info] = mussv(M_zv_g,[1,0;1,0;1,0;1,0;1,0],'a');
end

semilogx(omega,mubnds_rs(:,1).ResponseData(:),'-',...
    omega,mubnds_rs(:,2).ResponseData(:),'--','color',clr_lst(1,:)); 

% Mu for RS --> Parametric uncertainties only
if dparam
    
    M_zv = lftdata(M(1:length(Delta),1:length(Delta)));
    M_zv_dpert = frd(M_zv(1:2,1:2),omega);
    [mubnds,Info] = mussv(M_zv_dpert,[-1,0;-1,0],'a');
    
    semilogx(omega,mubnds(:,1).ResponseData(:),'-', ...
        omega,mubnds(:,2).ResponseData(:),'--','color', clr_lst(2,:)); 
end

% Mu for RS --> Delta_i only
M_zv_di = frd(M(1:2,1:2),omega);
[mubnds,Info] = mussv(M_zv_di,[1,0;1,0],'a');
semilogx(omega,mubnds(:,1).ResponseData(:),'-', ...
    omega,mubnds(:,2).ResponseData(:),'--','color', clr_lst(3,:)); 

% Mu for RS --> Delta_o only
M_zv_do = frd(M(3:length(Delta),3:length(Delta)),omega);
[mubnds,Info] = mussv(M_zv_do,[1,0;1,0;1,0],'a');

% Mu(=sigma_upper) for NP --> complete d vector
M_ed_g = frd(M(6:8,6:7),omega);
[mubnds,Info] = mussv(M_ed_g,[2,3],'a');
semilogx(omega,mubnds(:,1).ResponseData(:),'-', ...
    omega,mubnds(:,2).ResponseData(:),'--','color', clr_lst(end,:)); 

% Mu(=sigma_upper) for NP --> pilot input only
M_e_dp = frd(M(6:8,6),omega);
[mubnds,Info] = mussv(M_e_dp,[1,3],'a');
semilogx(omega,mubnds(:,1).ResponseData(:),'-',...
    omega,mubnds(:,2).ResponseData(:),'--','color', clr_lst(4,:)); 

% Mu(=sigma_upper) for NP --> gust disturbance only
M_e_dg = frd(M(6:8,7),omega);
[mubnds,Info] = mussv(M_e_dg,[1,3],'a');
semilogx(omega,mubnds(:,1).ResponseData(:),'-', ...
    omega,mubnds(:,2).ResponseData(:),'--','color', clr_lst(7,:)); 

% Plot mu = 1
semilogx(omega,ones(size(omega)),'k:');

% Axis limits
ylim([0,3.5]);

% Add legend
hlegend = legend('$\bar{\mu}_{RP}$','$\underline{\mu}_{RP}$', ...
                 '$\bar{\mu}_{RS}$','$\underline{\mu}_{RS}$', ...
                 '$\bar{\mu}_{RS_{\delta}}$','$\underline{\mu}_{RS_{\delta}}$', ... 
                 '$\bar{\mu}_{RS_{\Delta}}$','$\underline{\mu}_{RS_{\Delta}}$', ... 
                 '$\bar{\mu}_{NP}$','$\underline{\mu}_{NP}$', ... 
                 '$\bar{\mu}_{NP_r}$','$\underline{\mu}_{NP_r}$', ... 
                  '$\bar{\mu}_{NP_d}$','$\underline{\mu}_{NP_d}$', ... 
                 'Interpreter','latex','Location','NorthEast','FontSize',12);
             
 hlegend.NumColumns  = 4;
 hlegend.Orientation = 'horizontal';

% Add axis labels
xlabel('Frequency (rad/s)','Interpreter','latex','FontSize',12); 
ylabel('$\mu$ (-)','Interpreter','latex','FontSize',12); 

set(gca,'TickLabelInterpreter','latex')

% --- (3) M-analysis plot (reduced) ---
FIG_muMred = figure(3); FIG_muMred.Position = [681 559 560 380];

% Mu for RP
semilogx(omega,mubnds_rp(:,1).ResponseData(:),'k-', ...
    omega,mubnds_rp(:,2).ResponseData(:),'k--','LineWidth',0.5); 

grid on; hold on; xlim([omega(1),omega(end)]);

% Mu for RS
if dparam
    M_zv_g = frd(lftdata(M(1:length(Delta),1:length(Delta))),omega);
    [mubnds_rs,Info] = mussv(M_zv_g,[-1,0;-1,0;1,0;1,0;1,0;1,0;1,0],'a');
else
    M_zv_g = frd(M(1:length(Delta),1:length(Delta)),omega);
    [mubnds_rs,Info] = mussv(M_zv_g,[1,0;1,0;1,0;1,0;1,0],'a');
end

semilogx(omega,mubnds_rs(:,1).ResponseData(:),'-',...
    omega,mubnds_rs(:,2).ResponseData(:),'--','color',clr_lst(1,:),'LineWidth',0.5); 

% Mu(=sigma_upper) for NP --> complete d vector
M_ed_g = frd(M(6:8,6:7),omega);
[mubnds_np,Info] = mussv(M_ed_g,[2,3],'a');
semilogx(omega,mubnds_np(:,1).ResponseData(:),'-', ...
    omega,mubnds_np(:,2).ResponseData(:),'--','color', clr_lst(end,:),'LineWidth',0.5);

% Plot mu = 1
semilogx(omega,ones(size(omega)),'k:','LineWidth',0.5);

% Axis limits
ylim([0,3.5]);

% Add legend
hlegend = legend('$\bar{\mu}_{RP}$','$\underline{\mu}_{RP}$', ...
                 '$\bar{\mu}_{RS}$','$\underline{\mu}_{RS}$', ...
                 '$\bar{\mu}_{NP}$','$\underline{\mu}_{NP}$', ... 
                 'Interpreter','latex','Location','NorthEast','FontSize',12);
             
 hlegend.NumColumns  = 2;
 hlegend.Orientation = 'horizontal';

% Add axis labels
xlabel('Frequency (rad/s)','Interpreter','latex','FontSize',12); 
ylabel('$\mu$ (-)','Interpreter','latex','FontSize',12); 

set(gca,'TickLabelInterpreter','latex')

% Store peak mu
mu_bar_rp = [getPeakGain(mubnds_rp(:,1)),getPeakGain(mubnds_rp(:,2))];
mu_bar_rs = [getPeakGain(mubnds_rs(:,1)),getPeakGain(mubnds_rs(:,2))];
mu_bar_np = [getPeakGain(mubnds_np(:,1)),getPeakGain(mubnds_np(:,2))];

% --- (4) Performance robustness margins plot ---
FIG_muRPdec = figure(4); FIG_muRPdec.Position = [681 559 560 380];

semilogx(omega,mubnds_rp(:,1).ResponseData(:),'k-',omega, ...
            mubnds_rp(:,2).ResponseData(:),'k--','LineWidth',0.5); 
        
grid on; hold on; xlim([omega(1),omega(end)]);

% RP margins
M_g = frd(lftdata(M),omega);

for i = 1:length(M_zv_g)-3
    
    if dparam && i < 3
        pert = [-1,0];
    else
        pert = [1,0];
    end 
    
    if dparam     
        [mubnds,Info] = mussv(M_g([i,8:10],[i,8:9]),[pert;2,3],'a');        
    else    
        [mubnds,Info] = mussv(M_g([i,6:8],[i,6:7]),[pert;2,3],'a');        
    end
    
    semilogx(omega,mubnds(:,1).ResponseData(:),'-','color',clr_lst(i,:),'LineWidth',0.5);  
    semilogx(omega,mubnds(:,2).ResponseData(:),'--','color',clr_lst(i,:),'LineWidth',0.5); 
       
    if dparam && i == 4    
        
        [mubnds,Info] = mussv(M_g([i-1:i,8:10],[i-1:i,8:9]),[pert;pert;2,3],'a');
        semilogx(omega,mubnds(:,1).ResponseData(:),'-','color',clr_lst(end-1,:),'LineWidth',0.5);  
        semilogx(omega,mubnds(:,2).ResponseData(:),'--','color',clr_lst(end-1,:),'LineWidth',0.5); 
        
    elseif ~dparam && i == 2
        
        [mubnds,Info] = mussv(M_g([i-1:i,6:8],[i-1:i,6:7]),[pert;pert;2,3],'a');
        semilogx(omega,mubnds(:,1).ResponseData(:),'-','color',clr_lst(end-1,:),'LineWidth',0.5);  
        semilogx(omega,mubnds(:,2).ResponseData(:),'--','color',clr_lst(end-1,:),'LineWidth',0.5); 
        
    end
    
end

% Plot nominal performance
[mubnds,Info] = mussv(M_g([8:10],[8:9]),[2,3],'a');
semilogx(omega,mubnds(:,1).ResponseData(:),'-','color',clr_lst(end,:),'LineWidth',0.5);  
semilogx(omega,mubnds(:,2).ResponseData(:),'--','color',clr_lst(end,:),'LineWidth',0.5);  

% Plot mu = 1
semilogx(omega,ones(size(omega)),'k:','LineWidth',0.5);

% Axis limits
ylim([0,3.5]);

% Add legend
if dparam
    hlegend = legend('$\bar{\mu}_{RP}$','$\underline{\mu}_{RP}$', ...
                     '$\bar{\mu}_{RP_{\delta M_a}}$','$\underline{\mu}_{RP_{\delta M_a}}$',...
                     '$\bar{\mu}_{RP_{\delta M_{\delta_e}}}$', '$\underline{\mu}_{RP_{\delta M_{\delta_e}}}$', ...
                     '$\bar{\mu}_{RP_{\Delta_{act}}}$', '$\underline{\mu}_{RP_{\Delta_{act}}}$', ...
                     '$\bar{\mu}_{RP_{\Delta_{ba}}}$', '$\underline{\mu}_{RP_{\Delta_{ba}}}$', ...
                     '$\bar{\mu}_{RP_{\Delta}}$', '$\underline{\mu}_{RP_{\Delta}}$', ...
                     '$\bar{\mu}_{NP}$', '$\underline{\mu}_{NP}$', ...
                     'Interpreter','latex','Location','NorthEast','FontSize',12);
else
    
hlegend = legend('$\bar{\mu}_{RP}$','$\underline{\mu}_{RP}$', ...
                     '$\bar{\mu}_{RP_{\Delta_1}}$', '$\underline{\mu}_{\Delta_{i_1}}$', ...
                     '$\bar{\mu}_{RP_{\Delta_2}}$', '$\underline{\mu}_{RP_{\Delta_2}}$', ...
                     '$\bar{\mu}_{RP_{\Delta}}$', '$\underline{\mu}_{RP_{\Delta}}$', ...
                     '$\bar{\mu}_{NP}$', '$\underline{\mu}_{NP}$', ...
                     'Interpreter','latex','Location','NorthEast','FontSize',12);
end
             
hlegend.NumColumns  = 4;
hlegend.Orientation = 'horizontal';

set(gca,'TickLabelInterpreter','latex')

% Add axis labels
xlabel('Frequency (rad/s)','Interpreter','latex','FontSize',12); 
ylabel('$\mu$ (-)','Interpreter','latex','FontSize',12); 

% --- (5) Nominal tracking performance ---
FIG_step_nom = figure(5); FIG_step_nom.Position = [681 559 560 380];

t_lst = 0:0.01:2.5;

y_unc = step(Ma_unc.NominalValue([5,4],1)/Wcmd,t_lst); 
y_ref = step(H_qref(1),t_lst); grid on; 

sp1 = subplot(2,1,1);
plot(t_lst,y_ref(:,1),'k--'); grid on; hold on;
plot(t_lst,y_unc(:,1),'k-');
ylim([0,7]);
ylabel('Pitch rate (deg/s)','Interpreter','latex','FontSize',12);
set(gca,'TickLabelInterpreter','latex')
set(gca,'XTickLabel',[])

% Add legend
legend('Desired', 'Nominal', ...
    'Interpreter','latex','Location','NorthEast','FontSize',12);

sp1.Position = sp1.Position + [0 -0.05 0 0.05];

sp2 = subplot(2,1,2);
plot(t_lst,y_unc(:,2),'k-'); grid on; hold on;
ylim([0,5]);
xlabel('Time (s)','Interpreter','latex','FontSize',12);
ylabel('Angle-of-attack (deg)','Interpreter','latex','FontSize',12);
set(gca,'TickLabelInterpreter','latex')

sp2.Position = sp2.Position + [0 0 0 0.05];

% --- (6) Plot nominal broken loop transfer functions --- 
FIG_BL = figure(6); FIG_BL.Position = [681 559 560 380];

if any(ismember(claw_syn,3:6))

    % Combine FF and FB CLAW elements
    Kt = append(Kff,K);
    
    if claw_syn == 5 
        H_syn = [tf(1);tf(1)];
        H_syn.InputName = 'ufb0_t';
        H_syn.OutputName = {'ufb0','ufb0_syn'};
        
        u_sum = sumblk('ut = u + dde');
        
        Kt = connect(Kt,H_syn,u_sum,[Kt.InputName(1:end-2);'ufb0_t'],'ut');
        Pt.OutputName{end} = 'ufb0_t';
        
    else
        u_sum = sumblk('ut = u + uff');
        
        Kt = connect(Kt,u_sum,Kt.InputName,'ut');
    end
    
    Kt.OutputName = 'u';
    
    if claw_syn == 6
            
        if t_actfb_mdl
            H_act_obm = H_act;
        else
            H_act_obm = tf(1);
        end

        H_act_obm.InputName  = 'ufb0';
        H_act_obm.OutputName = 'ufb';    
            
        H_split = [-H_alloc,H_act_obm];
        H_split.InputName = {'qdots','ufb0'};
        H_split.OutputName = 'inv_fb';
        
        Kt = connect(Kt,H_split,u_sum,[Kt.InputName(1:end-1);'qdots';'ufb0'],'u');
        Pt = Pbt;
        
    end

    % (A) All loops broken
    if dparam
        L = -Kt*Pt.NominalValue(:,end);
    else
        L = -Kt*Pt(:,end);
    end
    
    % (B) Actuator feedback loop closed
    Lacl = act_connect(Pbt,Kt,H_aa,dparam);

else
    
    Kt = K;

    % (A) All loops broken
    if dparam
        L = -Kt*P.NominalValue(size(P.NominalValue,1)-nmeas+1:size(P.NominalValue,1),end);
    else
        L = -Kt*P(size(P,1)-nmeas+1:size(P,1),end);
    end
    
    % (B) Actuator feedback loop closed
    Lacl = act_connect(P,Kt,H_aa,dparam);

end

[mag_L,phase_L,~] = bode(L,omega); 
phase_L = phase_wrapper(phase_L(:));

[mag_Lacl,phase_Lacl,wout] = bode(Lacl,omega);
phase_Lacl = phase_wrapper(phase_Lacl(:));

sp1 = subplot(2,1,1);
semilogx(omega,mag2db(mag_L(:)),'k-'); grid on; hold on;
if ~strcmp(t_indi,'mb')
    semilogx(omega,mag2db(mag_Lacl(:)),'k--');
end
ylabel('Magnitude (dB)','Interpreter','latex','FontSize',12);
set(gca,'TickLabelInterpreter','latex')
set(gca,'XTickLabel',[])

sp1.Position = sp1.Position + [0 -0.05 0 0.05];

% Add legend
legend('All loops broken', 'Actuator loop closed', ...
    'Interpreter','latex','Location','NorthEast','FontSize',12);

sp2 = subplot(2,1,2);
semilogx(omega,phase_L,'k-'); grid on; hold on;
if ~strcmp(t_indi,'mb')
    semilogx(omega,phase_Lacl,'k--');
end
xlabel('Frequency (rad/s)','Interpreter','latex','FontSize',12);
ylabel('Phase (deg)','Interpreter','latex','FontSize',12);
set(gca,'TickLabelInterpreter','latex')

sp2.Position = sp2.Position + [0 0 0 0.05];

% --- (7) Bode plot of CLAW elements ---
FIG_CLAW = figure(7); FIG_CLAW.Position = [681 559 560 380];

bode(Kt,{omega(1),omega(end)},'k-'); grid on; hold on;

% Also plot full-order controller in case of post-synthesis order reduction
if fix_order >= 0 && strcmp(red_order,'postsyn')
    bode(Kfull,{omega(1),omega(end)},'k--'); 
end

% --- (8) Robust tracking performance ---
FIG_step = figure(8); FIG_step.Position = [681 559 560 380];

step_opts = timeoptions;
step_opts.YLimMode = 'manual'; step_opts.Ylim = [-1,7];

stepplot(Ma_unc([5,4],1)/Wcmd,0:0.001:4,step_opts); grid on; hold on;
stepplot(H_qref(1),0:0.01:4,step_opts,'k--');
stepplot(Ma_unc.NominalValue([5,4],1)/Wcmd,0:0.01:4,step_opts,'r-'); 

% --- (9) Bode plot of performance metric --- (unweighted tracking error)
FIG_WP = figure(9); FIG_WP.Position = [681 559 560 380];

h = bodeplot(Ma_unc(5,1)/Wcmd,{omega(1),omega(end)});
grid on; hold on;

p = getoptions(h); 
p.PhaseMatching = 'on'; 
p.Ylim{1} = [-30,20];
setoptions(h,p);

bodemag(Ma_unc.NominalValue(5,1)/Wcmd,{omega(1),omega(end)},'r-');
bodemag(H_qref(1)/Wcmd,{omega(1),omega(end)},'k--');

