%==========================================================================
% [main.m] Main file for (I)DI analysis and inversion filter synthesis
%
%     Performs mu-analysis and synthesis for structured (incremental) DI
%     controllers using the Robust Control Toolbox. This is done in the 
%     context of an F-16 short-period control design case study.
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

% Check if in batch mode
if ~exist('batch_mode','var')
    clear all; clc;
end

% Add paths
addpath(genpath('_FUNCTIONS/')); addpath('../_MODELS/');

% Misc
r2d = 180/pi;  % rad2deg
s   = tf('s'); % Laplace var

%% (USER) Analysis settings

% Model
fname_mdl = 'LINSS_F16_UOM_h10kft_V540fts_XCG0.382MAC.mat';
x_p       = 15;     % ft; pilot station
Vt0       = 540;    % ft/s
tau_act   = 60; 
H_act     = tf(tau_act,[1 tau_act]);

% Parametric uncertainty mode
dparam    = 1;

% Define parametric uncertainties
Delta_Ma  = 0.75; % +/- fraction
Delta_Mde = 0.25; % +/- fraction

% Set config ID
mdl_config = 'BS2';

% ---

% Desired dynamics
w_sp_d  = 4.0;   % rad/s
z_sp_d  = 1.0;   % - 

% Regulator settings
K_p     = 7; 
K_i     = 2;
H_fb    = K_p + 1/s*K_i;

% Set REG config ID
reg_config = 'BD';

% ---

% INDI and actuator feedback modes
t_indi        = 'sb';
t_actfb       = 'sb';
t_actfb_mdl   = 0; % Inactive in case t_actfb is selected as sb

% (Nominal CLAW) Inversion gain
KG_inv = 1;

% (Nominal CLAW) Sensed angular acceleration filter
H_aa_sb = tf(1); 

% (Nominal CLAW) Hybrid gain (1 equals SB, 0 equals MB)
zeta_h = sqrt(1/2); wn_h = 5;
Kh   = tf([wn_h^2],[1 2*zeta_h*wn_h, wn_h^2]); 
Khc  = 1 - Kh;

% Set INV config ID
inv_config = 'SS01AAA';

% Use external config ID in batch mode
if exist('batch_mode','var')
    inv_config = inv_config_ext;
end
    
% ---

% Synthesis settings
claw_syn      = 0;

mudisp        = 'short'; % 'full'; 'short';
mixedmu_syn   = 'on'; 
fix_order     = -1; % Filter order
fix_form      = 0;
red_order     = 'presyn'; % 'postsyn'; 'presyn';
max_iter      = 30;
Nsyn          = 3;
omega_syn     = logspace(-2,3,71);
Astruct       = 'full';
parallel      = false;

% Set SYN config ID
syn_config = strcat(num2str(claw_syn),'A');

% Save figures
save_data = false;
res_dir   = '_RESULTS/';

%% (USER) Weights

% Pilot stick command filter
Wcmd = tf(1);

% Input uncertainty (1) weight (act)
Wunci_1 = 2*tf([1 5],[1 100]);

% Input uncertainty (2) weight (ba)
Wunci_2 = 2*tf([1 5],[1 100]);

% Output uncertainty weights
Wunco_nz    = tf(0);
Wunco_q     = tf(0);
Wunco_qdot  = tf(0);

% Tracking error weights
Wte_nz    = tf(0);
Wte_q     = minreal(gen_wpte(1,50,30,2e-1,30));
Wte_qdot  = tf(0);

% Disturbance weight - Scaled with trim airspeed
Wdist = r2d/Vt0*tf(4,[1 0.2]);

% Set WGT config ID
wgt_config = '77ARA';

%% Intermediate batch mode logic
if exist('batch_mode','var') && strcmp(t_indi,'hb')
    H_aa_sb = tf(1); 
end

%% Create (uncertain) short-period system formulation

% Load model
load(fname_mdl);

% Extract short-period dynamics
ba_sp_sys = ss(ba_lon_sys.A(3:4,3:4),ba_lon_sys.B(3:4), ...
                    ba_lon_sys.C(3:6,3:4),ba_lon_sys.D(3:6));

% Add input disturbance input column
ba_sp_sys = ss(ba_sp_sys.A,[ba_sp_sys.B,-ba_sp_sys.A(:,1)], ...
                    ba_sp_sys.C,[ba_sp_sys.D,-[1;0;0;0]]);

% Preliminaries
g     = 32.1;   % ft/s^2
T_th2 = -1/zero(ba_sp_sys(2,1)); 
K_q   = dcgain(ba_sp_sys(2,1)); 
                
% Calculate ICR
x_icr = ba_sp_sys.B(1,1)*Vt0/(ba_sp_sys.B(2,1));

% Get nz at pilot station
ba_sp_sys.C(3,:) = ba_sp_sys.C(3,:) + x_p/g/r2d*ba_sp_sys.C(4,:);
ba_sp_sys.D(3,:) = ba_sp_sys.D(3,:) + x_p/g/r2d*ba_sp_sys.D(4,:);

% Parametric (real) uncertainties; 
if dparam

    Ma  = ureal('Ma',ba_sp_sys.A(2,1),'Percentage',[-Delta_Ma,Delta_Ma]*100);
    Mde = ureal('Mde',ba_sp_sys.B(2,1),'Percentage',[-Delta_Mde,Delta_Mde]*100);

    A_ba_sp = [ba_sp_sys.A(1,:); Ma ba_sp_sys.A(2,2)];
    B_ba_sp = [ba_sp_sys.B(1,:); Mde -Ma];    
    C_ba_sp = [ba_sp_sys.C(1:2,:); ba_sp_sys.C(3,:) + ...
                    x_p/g/r2d*A_ba_sp(2,:); A_ba_sp(2,:)];
    D_ba_sp = [ba_sp_sys.D(1:2,:); ba_sp_sys.D(3,:) + ...
                    x_p/g/r2d*B_ba_sp(2,:); B_ba_sp(2,:)];

    ba_sp_sys = ss(A_ba_sp,B_ba_sp,C_ba_sp,D_ba_sp);
end

%% Desired dynamics
T_th2_d = 1.0; %T_th2;

% Desired dynamics/load factor response type (DC gain nz = 1g)
H_qref  = w_sp_d^2*tf([T_th2_d 1],[1 2*z_sp_d*w_sp_d w_sp_d^2])/(Vt0/g/r2d);
H_nzref = (H_qref*(Vt0/g/r2d))*tf([1],[T_th2_d 1]);

%% Nominal 2DOF INDI CLAW elements

% Add derivative term to pitch rate command filter for feedforward
H_qref = [H_qref;s*H_qref];

% Angular acceleration estimation scheme
H_aa = [H_aa_sb,0,0,0]; % Sensor-based 

if dparam
    D_aa = [0,ba_sp_sys.NominalValue.C(4,:), ...
            ba_sp_sys.NominalValue.D(4,1)];
else
    D_aa = [0,ba_sp_sys.C(4,:), ba_sp_sys.D(4,1)];        
end

if strcmp(t_indi,'mb') || any(ismember(claw_syn,[4,6]))
    
    H_aa = ss(0,zeros(1,4),0,D_aa); 
    
elseif strcmp(t_indi,'hb')
    
    H_aa = ss([Kh*H_aa(1),(1-Kh)*tf(D_aa(2:end))]);
    
end

% Actuator feedback signal correction
if strcmp(t_actfb,'sb')
    t_actfb_mdl   = 0;
end

% Synchronization
H_aa_syn = H_aa;

if t_actfb_mdl
    H_aa_syn(1) = H_aa_syn(1)*H_act;
end

% Synchronization scheme
if strcmp(t_indi,'sb')
    H_syn = H_aa_syn(1); % Sensor-based      
elseif strcmp(t_indi,'mb')
    H_syn = tf([1]);
elseif strcmp(t_indi,'hb')
    H_syn = H_aa_syn(1) + H_aa_syn(4)/D_aa(4);
end

% Control allocation
if dparam
    H_alloc = tf([1/ba_sp_sys.NominalValue.D(4,1)])*KG_inv;
else
    H_alloc = tf([1/ba_sp_sys.D(4,1)])*KG_inv;    
end 

%% Create generalized plant description

H_act.InputName        = 'yact_in';
H_act.OutputName       = 'yact';
ba_sp_sys.InputName    = {'de','ag'};
ba_sp_sys.OutputName   = {'alpha','q','nz','qdot'};
Wcmd.InputName         = 'w(1)';
Wcmd.OutputName        = 'cmd';
H_qref.InputName       = 'cmd';
H_qref.OutputName      = {'yqref','yqdotref'};
H_nzref.InputName      = 'nzcmd';
H_nzref.OutputName     = 'ynzref';
Wte_q.InputName        = 'qerr';
Wte_q.OutputName       = 'yqte';
Wte_nz.InputName       = 'nzerr';
Wte_nz.OutputName      = 'ynzte';
Wte_qdot.InputName     = 'qdot_err';
Wte_qdot.OutputName    = 'yqdotte';
Wunci_1.InputName      = 'u';
Wunci_1.OutputName     = 'ydel(1)';
Wunci_2.InputName      = 'yact';
Wunci_2.OutputName     = 'ydel(2)';
Wunco_q.InputName      = 'q';
Wunco_q.OutputName     = 'ydel(3)';
Wunco_nz.InputName     = 'nz';
Wunco_nz.OutputName    = 'ydel(4)';
Wunco_qdot.InputName   = 'qdot';
Wunco_qdot.OutputName  = 'ydel(5)';
Wdist.InputName        = 'w(2)'; 
Wdist.OutputName       = 'ag';

sum_yact     = sumblk('yact_in = u + udel(1)');
sum_de       = sumblk('de = yact + udel(2)');
sum_qs       = sumblk('qs = q + udel(3)');
sum_alphas   = sumblk('alphas = alpha + udel(4)');
sum_qdots    = sumblk('qdots = qdot + udel(5)');
sum_qerr     = sumblk('qerr = yqref-q');
sum_nzerr    = sumblk('nzerr = ynzref-nz');
sum_qdot_err = sumblk('qdot_err = yqdotref-qdot');

if strcmp(t_actfb,'sb')    
    sum_ufb0  = sumblk('ufb0 = yact');
elseif strcmp(t_actfb,'mb')    
    sum_ufb0  = sumblk('ufb0 = u');
end

inputs  = {'udel','w','u'};
outputs = {'ydel','yqte','ynzte','yqdotte','yqref','yqdotref','alphas','qs','qdots','ufb0'};

nmeas = 6; nu = 1;

% Synthesis version
P = minreal(connect(H_act, ba_sp_sys, H_qref, H_nzref, Wcmd, Wte_nz, Wte_q, ...
               Wte_qdot, Wunci_1, Wunci_2, Wunco_q, Wunco_nz, Wunco_qdot, Wdist, sum_yact, ...
               sum_de, sum_qs, sum_alphas, sum_qdots, sum_qerr, sum_nzerr, sum_qdot_err, ...
               sum_ufb0, inputs, outputs));
               
% Analysis version
outputs_a = {'ydel','yqte','ynzte','yqdotte','alpha','q','nz','qdot','yqref','yqdotref', ...
            'alphas','qs','qdots','ufb0'};

Pa = minreal(connect(H_act, ba_sp_sys, H_qref, H_nzref, Wcmd, Wte_nz, Wte_q,...
               Wte_qdot, Wunci_1, Wunci_2, Wunco_q, Wunco_nz, Wunco_qdot, Wdist, sum_yact, ...
               sum_de, sum_qs, sum_alphas, sum_qdots, sum_qerr, sum_nzerr, sum_qdot_err, ...
               sum_ufb0, inputs, outputs_a));

% Store open-loop system
P0 = P; Pa0 = Pa;
           
% Generate IDI interconnection structure;
[~,Kinit,Kff,nmeas,nu,P,Pt,Pa,Pbt,outputs_a] = ...
    gen_interconnect(claw_syn,t_indi,t_actfb_mdl,P,Pa,H_fb,H_aa,H_syn, ...
                        H_alloc,H_act,inputs,outputs_a);

%% Synthesize controller

% Initialize
omega = omega_syn;

% Define Delta block (uncertainty only, performance is handled automatically)
Delta = append(ultidyn('Delta_i1',[1,1]), ultidyn('Delta_i2',[1,1]), ultidyn('Delta_oq',[1,1]), ...
                ultidyn('Delta_onz',[1,1]), ultidyn('Delta_oqdot',[1,1]));

% Generate uncertain generalized plant
Punc = lft(Delta,P);

% Perform mu-synthesis
if claw_syn

    opt = musynOptions('FrequencyGrid',omega,'MaxIter',max_iter, ...
                        'Display',mudisp,'MixedMU',mixedmu_syn, ...
                        'RandomStart',Nsyn-1,'UseParallel',parallel);

    if fix_order >= 0 && strcmp(red_order,'presyn')

        % Reduce controller order before synthesis (limit tunable parameters)
        K = tunableSS('K',fix_order,nu,nmeas,Astruct);
        
        if fix_order == 2 && fix_form        
            wf = realp('K',20); K = tf(1,[1/wf^2 2*sqrt(1/2)/wf 1]);            
        end  
        
        [Munc,CLperf,syn_info] = musyn(lft(Punc,K),opt);        
        K = getBlockValue(Munc,'K');   
        
        if fix_order == 2 && fix_form
            K = ss(tf(1,[1/K^2 2*sqrt(1/2)/K 1]));            
        end  

    else
        [K,CLperf,syn_info] = musyn(Punc,nmeas,nu,opt);
    end
    
else
    K = Kinit;

end

% Assign names
K.InputName  = P.OutputName(end-nmeas+1:end);
K.OutputName = {'u'};

% Store synthesized controller
Kfull = K;

% Reduce controller order after synthesis
if fix_order >= 0 && strcmp(red_order,'postsyn')
    K = reduce(K,fix_order);
end

% Assign names
K.InputName  = P.OutputName(end-nmeas+1:end);
K.OutputName = {'u'};

% Store controller for later scrutinization if needed
Ks = K;

%% Generate results
gen_results;

%% Export results

res_dir = strcat(res_dir,'DATA_',mdl_config,'_',reg_config, ...
    '_',inv_config,'_',wgt_config,'_',syn_config,'/');

if save_data
    
    fprintf('\nSaving data...\n\n');
    
    mkdir(res_dir);
    
    %% Save figures
    saveas(FIG_muRP,strcat(res_dir,'fig_muRP.fig'));
    saveas(FIG_muRP,strcat(res_dir,'fig_muRP.png'));
    saveas(FIG_muRP,strcat(res_dir,'fig_muRP.emf'));
    saveas(FIG_muRP,strcat(res_dir,'fig_muRP'),'epsc');
    
    saveas(FIG_muM,strcat(res_dir,'fig_muM.fig'));
    saveas(FIG_muM,strcat(res_dir,'fig_muM.png'));
    saveas(FIG_muM,strcat(res_dir,'fig_muM.emf'));
    saveas(FIG_muM,strcat(res_dir,'fig_muM'),'epsc');
    
    saveas(FIG_muMred,strcat(res_dir,'fig_muMred.fig'));
    saveas(FIG_muMred,strcat(res_dir,'fig_muMred.png'));
    saveas(FIG_muMred,strcat(res_dir,'fig_muMred.emf'));
    saveas(FIG_muMred,strcat(res_dir,'fig_muMred'),'epsc');
    
    saveas(FIG_muRPdec,strcat(res_dir,'FIG_muRPdec.fig'));
    saveas(FIG_muRPdec,strcat(res_dir,'FIG_muRPdec.png'));
    saveas(FIG_muRPdec,strcat(res_dir,'FIG_muRPdec.emf'));
    saveas(FIG_muRPdec,strcat(res_dir,'FIG_muRPdec'),'epsc');
    
    saveas(FIG_step,strcat(res_dir,'fig_step.fig'));
    saveas(FIG_step,strcat(res_dir,'fig_step.png'));
    saveas(FIG_step,strcat(res_dir,'fig_step.emf'));
    saveas(FIG_step,strcat(res_dir,'fig_step'),'epsc');
    
    saveas(FIG_step_nom,strcat(res_dir,'fig_step_nom.fig'));
    saveas(FIG_step_nom,strcat(res_dir,'fig_step_nom.png'));
    saveas(FIG_step_nom,strcat(res_dir,'fig_step_nom.emf'));
    saveas(FIG_step_nom,strcat(res_dir,'fig_step_nom'),'epsc');
    
    saveas(FIG_BL,strcat(res_dir,'fig_BL.fig'));
    saveas(FIG_BL,strcat(res_dir,'fig_BL.png'));
    saveas(FIG_BL,strcat(res_dir,'fig_BL.emf'));
    saveas(FIG_BL,strcat(res_dir,'fig_BL'),'epsc');
    
    saveas(FIG_CLAW,strcat(res_dir,'fig_CLAW.fig'));
    saveas(FIG_CLAW,strcat(res_dir,'fig_CLAW.png'));
    saveas(FIG_CLAW,strcat(res_dir,'fig_CLAW.emf'));
    saveas(FIG_CLAW,strcat(res_dir,'fig_CLAW'),'epsc');
    
    saveas(FIG_WP,strcat(res_dir,'fig_WP.fig'));
    saveas(FIG_WP,strcat(res_dir,'fig_WP.png'));
    saveas(FIG_WP,strcat(res_dir,'fig_WP.emf'));
    saveas(FIG_WP,strcat(res_dir,'fig_WP'),'epsc');
    
    %% Export umbrella

    % Get git commit hash
    [s,git_hash] = system('git rev-parse HEAD');

    % Analysis settings
    mdl = struct('fname_mdl',fname_mdl,'ba_sp_sys',ba_sp_sys,'x_p',x_p,'Vt0',Vt0,'tau_act', ...
        tau_act,'dparam',dparam,'Delta_Ma',Delta_Ma,'Delta_Mde',Delta_Mde);

    reg = struct('w_sp_d',w_sp_d,'z_sp_d',z_sp_d,'T_th2_d',T_th2_d, ...
        'K_p',K_p,'K_i',K_i);

    syn = struct('mixedmu_syn',mixedmu_syn,'fix_order',fix_order,'fix_form',fix_form,....
         'red_order',red_order,'max_iter',max_iter,'Nsyn',Nsyn,'omega_syn', ...
         omega_syn,'Astruct',Astruct);
    inv = struct('H_aa_sb',H_aa_sb,'Kh',Kh,'KG_inv',KG_inv);

    settings = struct('mdl',mdl,'reg',reg,'t_indi',t_indi,'t_actfb',t_actfb, ...
        't_actfb_mdl',t_actfb_mdl,'claw_syn',claw_syn,'inv',inv,'syn',syn);

    % Weights
    weights = struct('H_qref',H_qref,'Wcmd',Wcmd,'Wunci_1',Wunci_1,'Wunci_2',Wunci_2,'Wunco_nz', ...
        Wunco_nz,'Wunco_q',Wunco_q,'Wunco_qdot',Wunco_qdot,'Wte_nz',Wte_nz, ...
        'Wte_q',Wte_q,'Wte_qdot',Wte_qdot,'Wdist',Wdist);

    % Systems
    systems = struct('Delta',Delta,'P',P,'Pa',Pa,'Pt',Pt,'Pbt',Pbt,...
        'K',K,'Kff',Kff,'Kt',Kt,'H_aa',H_aa,'H_alloc',H_alloc);

    % Overview
    info = struct('time_stamp',datetime('now'),'git_hash',git_hash);
    
    % Results
    results = struct('mu_bar_rp',mu_bar_rp,'mu_bar_rs',mu_bar_rs, ...
        'mu_bar_np',mu_bar_np);

    % Bring under umbrella struct
    umbrella = struct('info',info,'settings',settings,'weights',weights, ...
        'systems',systems,'results',results);

    if exist('syn_info','var')
       umbrella.syn_info = syn_info;
    end

    % Save
    export_fname = 'umbrella.mat';
    save(strcat(res_dir,export_fname),'umbrella');
end

fprintf('\n****** EOF *****\n');
