%==========================================================================
% [ssidi.m] Closed-loop IDI state-space formulation and analysis
%
%     Generates the state-space descriptions as derived in Section 2.1.3
%     'Linear state space insights'. File naming is in agreement with the
%     notation presented in the text. Workspace insights can be compared to
%     the model in _SIMULINK/ssidi_mdl.slx.
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

clear all; close all; clc;

%% Preliminaries

% Add path
addpath('../_MODELS/');

% Load model
fname_mdl = 'LINSS_F16_UOM_h10kft_V540fts_XCG0.382MAC.mat';
load(fname_mdl);

% Set plant state-space
Ap = ba_lon_sys.A; Bp = ba_lon_sys.B; 
Cp = ba_lon_sys.C(4,:); Dp = ba_lon_sys.D(4); 

% Convert plant state-space to controllable canonical form
n = length(Ap); m = n-1;

sysp  = ss(Ap,Bp,Cp,Dp);
syspc = canon(sysp,'companion');
tfp   = minreal(tf(sysp));

Apc = syspc.A';
Bpc = zeros(n,1); Bpc(n) = 1;

[nump,denp] = tfdata(tfp);
Cpc = flip(nump{1}); Cpc = Cpc(1:end-1); Dpc = 0;

% Transform state-space
TT = [Cpc; eye(m), zeros(m,n-m)];

TAT = TT*Apc*inv(TT); TBp = TT*Bpc; CBp = TBp(1);
R = TAT(1:n-m,1:n-m); T = TAT(1:n-m,n-m+1:n); PQ = TAT(n-m+1:n,:);

% Set model control effectiveness
K_B = 1; CBm = K_B*CBp;

% Get perturbation state-space description: u feedback loop
wn_1 = 80; Delta_1 = ss(tf(wn_1^2,[1 2*sqrt(1/2)*wn_1 wn_1^2])) - 1;
A1 = Delta_1.A; B1 = Delta_1.B; C1 = Delta_1.C; D1 = Delta_1.D; 

% Get perturbation state-space description: y_dot feedback loop
wn_2 = 40; Delta_2 = ss(tf(wn_2^2,[1 2*sqrt(1/2)*wn_2 wn_2^2])) - 1;
A2 = Delta_2.A; B2 = Delta_2.B; C2 = Delta_2.C; D2 = Delta_2.D; 

%% State-space formulations

% Initialize special variables
I = eye(size(B1,2)); Si = inv(I - CBm*D1*inv(CBp)+D2);

% Create A, B, C, D matrices as per Eq. (2.22)
A = [-Si*CBm*D1*inv(CBp)*R, -Si*CBm*D1*inv(CBp)*T, Si*CBm*C1, -Si*C2    ;
     PQ, zeros(m,length(A1)+length(A2))                                 ;
     -B1*inv(CBp)*(Si*CBm*D1*inv(CBp)+I)*R, ...
      B1*inv(CBp)*(Si*CBm*D1*inv(CBp)+I)*T, ...
      B1*inv(CBp)*Si*CBm*C1+A1, ...
     -B1*inv(CBp)*Si*C2                                                 ;
     -B2*Si*CBm*D1*inv(CBp)*R, ...
     -B2*Si*CBm*D1*inv(CBp)*T, ...
      B2*Si*CBm*C1, ...
      -B2*Si*C2+A2                                                      ];
B = [Si; zeros(size(PQ,1),1); Si*B1*inv(CBp); Si*B2];
C = A(1,:); D = B(1,:);

% Create state-space object
ss_idi = ss(A,B,C,D);

% Set synchronizatin dynamics as per Eq. (2.28)
A_psi = A(end-length(A1)-length(A2)+1:end,end-length(A1)-length(A2)+1:end);
A_psi_bar = [A1+B1*C1 -K_B*B1*C2; B2*C1 A2-K_B*B2*C2];

% Analyze poles
damp(A_psi)

%% Generate plots

% Bode diagram (nu --> y_dot)
figure; bode(ss_idi); grid on;

% Step response (nu --> y_dot)
figure; step(ss_idi,5); grid on;
